/* abc.h - Adaptive Block Coder */
/* 01/19/2008, written by Yuriy A. Reznik <yreznik@ieee.org> */

/* This software and documentation is provided "as is", and the author makes 
 * no representations or warranties, expressed or implied, including, but not
 * limited to, warranties of merchantability or fitness for any particular
 * purpose, or that the use of this software or documentation will not infringe 
 * any third parties patents, copyrights, trademarks or other rights. The 
 * author will not be liable for any direct, indirect, special or consequential 
 * damages arising out of any use of the software or documentation. */

#ifndef _ABC_H_
#define _ABC_H_   1

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * Block code length / Maximum number of blocks used as contexts
 *  - currently set to 8 and 3 correspondingly; 
 *  - changing these parameters will require regeneration of code tables in file abc.c
 *  - achievable redundancy rate with such settings: 
 *     ~ 1/2N log (N+B*N)/(B*N) ~ 0.0259 [bits / bit] (closer to 3% in practice)
 */
#define ABC_N 8
#define ABC_B 3

/*  
 * Initialize ABC encoder/decoder:
 */
void abc_enc_init (void);
void abc_dec_init (void);

/*
 * ABC encoder
 * Input:
 *  block - input sequence of ABC_N bits to encode
 *  cx_blocks - number of prior blocks used as a context (1..3, 0 -> no context)
 *  cx_count - number of bits set to one in prior blocks
 *  bstrm - bitstream structure
 * Returns: 
 *  number of bits set to one in encoded block
 */
unsigned abc_enc (unsigned block, unsigned cx_blocks, unsigned cx_count, BITSTREAM *bstrm);

/*
 * ABC decoder:
 * Input:
 *  block - pointer to a decoded block 
 *  cx_blocks - number of prior blocks used as a context (1..3, 0 -> no context)
 *  cx_count - number of bits set to one in prior blocks
 *  bstrm - bitstream structure
 * Returns: 
 *  number of bits set to one in decoded block
 */
unsigned abc_dec (unsigned *block, unsigned cx_blocks, unsigned cx_count, BITSTREAM *bstrm);

#ifdef __cplusplus
}
#endif

#endif /* _ABC_H_ */


