/* bitstream.h - simple bitstream IO functions */
/* 01/19/2008, written by Yuriy A. Reznik <yreznik@ieee.org> */

/* This software and documentation is provided "as is", and the author makes 
 * no representations or warranties, expressed or implied, including, but not
 * limited to, warranties of merchantability or fitness for any particular
 * purpose, or that the use of this software or documentation will not infringe 
 * any third parties patents, copyrights, trademarks or other rights. The 
 * author will not be liable for any direct, indirect, special or consequential 
 * damages arising out of any use of the software or documentation. */

#ifndef _BITSTREAM_H_
#define _BITSTREAM_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

/* bitstream structure: */
typedef struct {
  unsigned char *p;       /* current position  */
  unsigned int bb;        /* bit buffer (left-justified) */
  int offset;             /* # of bits in the bit buffer */
  unsigned char *start_p; /* start bitstream position */
  int start_offset;       /* offset in the first byte in the bitstream */
  int mode;               /* mode: 0 -> reading; 1 -> writing  */
} BITSTREAM;

/* open/close bitstream: */
void bitstream_open (BITSTREAM *p, unsigned char *ps, int offset, int write);
unsigned int bitstream_close (BITSTREAM *p); /* returns total number of bits read/written */

#define BITSTREAM_MAX_LEN 24  /* maximum bit-vector length supported by read/write() functions */

/* bitstream read/write functions: */
unsigned bitstream_read (BITSTREAM *p, int len);
void bitstream_write (BITSTREAM *p, unsigned bits, int len);

/* functions supporting fast VLC decoding: */
#define bitstream_buffer(p) (p->bb)             /* show bit-buffer */
void bitstream_scroll (BITSTREAM *p, int len);  /* scroll bitstream by len bits */

#ifdef __cplusplus
}
#endif

#endif /*_BITSTREAM_H */

